#! /usr/bin/env python3

import datetime
from dateutil.relativedelta import relativedelta
import prettytable
import sqlite3


class PPPCounter(object):
    def __init__(self, dbpath):
        self._conn = sqlite3.connect(dbpath,
                                     detect_types=(sqlite3.PARSE_DECLTYPES |
                                                   sqlite3.PARSE_COLNAMES))
        sqlite3.dbapi2.converters['DATETIME'] = sqlite3.dbapi2.converters['TIMESTAMP']

        self._create_table()

    def _exec_query(self, query, params=()):
        cur = self._conn.cursor()
        res = cur.execute(query, params)
        self._conn.commit()

        return res.fetchall()

    def _create_table(self):
        sql = "CREATE TABLE IF NOT EXISTS ppphistory (" \
              "  endtime DATETIME PRIMARY KEY," \
              "  sendbytes INTEGER," \
              "  recvbytes INTEGER," \
              "  conntime  INTEGER);"

        self._exec_query(sql)

    def insert(self, sendbytes, recvbytes, conntime, timestamp=None):
        if not timestamp:
            tmp = datetime.datetime.now()
            timestamp = tmp.replace(microsecond=0)

        sql = "INSERT INTO ppphistory(endtime, sendbytes, recvbytes, conntime)" \
              "  VALUES (?, ?, ?, ?)"
        params = (timestamp, sendbytes, recvbytes, conntime)

        return self._exec_query(sql, params)

    def vacuumdb(self):
        sql = "VACUUM"
        self._exec_query(sql)

    def clean(self):
        sql = "DELETE FROM ppphistory"
        self._execute_query(sql)
        self.vacuumdb()

    def delete_old_history(self, timestamp):
        sql = "DELETE FROM ppphistory" \
              "  WHERE endtime < ?"
        params = (timestamp,)

        self._exec_query(sql, params)
        self.vacuumdb()

    def _first_day_datetime(self):
        now = datetime.datetime.now()
        first_day = now.replace(day=1, hour=0, minute=0, second=0, microsecond=0)

        return first_day

    def delete_before_prev_month(self):
        first_day = self._first_day_datetime()
        ref_day = first_day - relativedelta(months=1)
        self.delete_old_history(ref_day)

    def amount_of_data(self, prev=False):
        first_day = self._first_day_datetime()

        if prev:
            start = first_day - relativedelta(months=1)
            end = first_day
        else:
            start = first_day
            end = first_day + relativedelta(months=1)

        sql = "SELECT SUM(sendbytes), SUM(recvbytes) FROM ppphistory" \
              "  WHERE endtime >= ? and endtime < ?"
        tmp = self._exec_query(sql, (start, end))[0]

        res = {"send" : tmp[0] if tmp[0] else 0,
               "recv" : tmp[1] if tmp[1] else 0}

        return res

    def select_all(self):
        sql = "SELECT * FROM ppphistory ORDER BY endtime"
        res = self._exec_query(sql)

        return res

    def select_data(self, prev=False):
        first_day = self._first_day_datetime()

        if prev:
            start = first_day - relativedelta(months=1)
            end = first_day
        else:
            start = first_day
            end = first_day + relativedelta(months=1)

        sql = "SELECT endtime, sendbytes, recvbytes, conntime FROM ppphistory" \
              "  WHERE endtime >= ? and endtime < ? order by endtime"
        rows = self._exec_query(sql, (start, end))

        return rows

    def get_table(self, prev=False, html=False):
        rows = self.select_data(prev)

        table = prettytable.PrettyTable()
        table.field_names = ["EndTime", "SendBytes", "RecvBytes", "ConnTime"]

        for row in rows:
            table.add_row(row)

        if html:
            return table.get_html_string()
        else:
            return table.get_string()
